      program strain
*
*************************************************************************
*                                                                       *
*   This program creates the files containing the strain data           *
*   for programm transq versions sequential and parallel                *  
*                                                                       *
*   Y. Epelboin                           March 1995                    *
*                                         April 1996                    *
*                                                                       *
*   version 2 for deformation model as written by J. Detaint            *
*   version 3 for generalized deformation and output for transq         *
*             sequential and parallel versions                          *
*                                                                       *
*   Version for Moire (ESRF) see V-49                                   *
*                                                                       *
*                                                                       *
*************************************************************************
*
      implicit none
      include 'strain.h'
*
      integer SIZE
      parameter (SIZE=30)
      real strain_val(x_max,2*z_max)
      character * 40 filein
      character *30 filestr      ! Modify SIZE if changed
      character *32 file_1       ! SIZE + 2
      character *34 file_2       ! SIZE + 4
      integer line_min,line_max,processor,i,j,ir
      integer strain_create
      integer dimz,dimx,ix,iz,line,offset,xmax  
      integer defaut, out(proc_max,2)
      real epais
      real stepx,stepz,stepy,xdeb,xfin
      real stepmx, stepmz
*
*     I/O units:  
*                log: logfile
*                ppp: strain data parallel
*                sss: strain data sequential
*                iin: strain files
*
      integer iin,log,ppp,sss
      parameter (iin=10,log=20,ppp=25,sss=26)
*    
*     data for defect (must be adapted to each case)
*     Definition: see function defaut
*
*     Variables for communication between function defaut
*     (defect initialization) and function strain_create
*     (defect calculation)
*

*     data 
*
*     strain_val:         array for du/dx, du/dz
*     line_min, line_max: first and last line to compute
*     epais:              thickness of crystal
*     xdeb:               position along surface where to
*                         start calculation.
*                         0 means starts at the origin
*                         > 0 computes part of the device only 
*     xfin:               position where to end the calculation
*                         (see V-38)
*
      write(*,*) 'Name of data file?'
      read(*,'(A)') filein
      open (unit=15,file=filein,iostat=ir)
      if (ir.ne.0) then
          write(*,*) 'Error opening data file ',filein
          stop 'error open data file'
      endif
      read(15,'(A)') filestr
      read(15,*) line_min,line_max
      processor = line_max - line_min +1
      if (processor.gt.proc_max) then
          write(*,*) 'Warning. Too much lines for image '
          write(*,*) 'Maximum possible ',proc_max
          write(*,*) 'Number asked ',processor
          write(*,*) 'check header strain.h and recompile'
      endif
      read(15,*) stepz,stepx,stepy
      read(15,*) xdeb,xfin

      open (unit=log,file='strain.log')
      write(*,1000)filestr,line_min,line_max,stepx,
     &      stepz,stepy,xdeb,xfin
      write(log,1000)filestr,line_min,line_max,stepx,
     &      stepz,stepy,epais,xdeb,xfin
1000  format('strain  Version 3.0 03/96'///
     &       'Characteristics of strain files'/
     &       '-------------------------------'/
     &       'Generic file name          ',A/
     &       'First and last lines       ',I4,' ',I4/
     &       'Step (x,z,y) in microns    ',F7.1,' ',F7.1,' ',F7.1/   
     &       'Starting point x (microns) ',F7.1/
     &       'Last point x (microns)     ',F7.1//
     &       '-------------------------------'/
     &       'Characteristics of defect'/
     &       '----------------------------'/) 
*
*     conversion of data into SI
*        
      stepmx=stepx
      stepmz=stepz
      stepx=stepx*1.e-06
      stepy=stepy*1.e-6
      stepz=stepz*1.e-6
      xdeb=xdeb*1.e-6
      xfin=xfin*1.e-6 
*
*     defect data
*     Initialization for defect
*     to be adapted to each case: 
*     Exchange data with strain_create through common /defect/
*     Return value of epais (thickness of crystal)
*
      ir = defaut(log,stepx,stepy,stepz,epais,xdeb,xfin)
      if (ir.ne.0) then
          write(*,*) 'Error defect initialization: ',ir
          stop 'Error defect initialization'
      endif    
      close (unit=15)   
*
*   build image name
*
      do j = SIZE,1,-1             
          if(filestr(j:j).ne.' ') goto 1
      enddo
      stop 'Error generic file name to short'
1     continue
      write(unit=file_2,fmt=2000)filestr(1:j),'.img'
2000      format(A,A4)
*
*     Check data
*
      if (xfin.le.0.) then
          write(*,*) 'Observation field (xfin) incorrect'
          write(*,*) 'xfin = ',xfin
          stop 'Error xfin'
      endif  
      if(xfin.le.xdeb) then
          write(*,*) 'xfin < xdeb: observation field incorrect'
          write(*,*) 'xfin = ',xfin,' xdeb = ',xdeb
          stop 'xfin < xdeb'
      endif
*     control for dimensions of strain_val (derivatives array)
*
      dimz = int(epais/stepz)+1
      if(dimz.gt.z_max) then
          write(*,*) 'dimz = ',dimz,' z_max = ',z_max
          write(*,*) 'modify header strain.h and recompile'
          stop 'abnormal end dimz'
      endif
      xmax = int((xfin-xdeb)/stepx) + 1
      if(xmax.gt.x_max) then
          write(*,*) 'dimx (max) = ',xmax, ' x_max = ',x_max
          write(*,*) 'Modify header strain.h and recompile'
          stop 'Error x_max'
      endif
      if(stepy.eq.0..and.processor.gt.1) then
          write(*,*) 'incoherent values between line number = ',
     &    processor
          write(*,*) 'and step y = 0.'
          stop 'abnormal end stepy'
      endif
*     
*     open output files for transqs and transqp formats
*
      open(unit=ppp,file='transq_p.dat')
      open(unit=sss,file='transq_s.dat')   
      write(sss,'(A)') filestr
      write(sss,1020) line_min,line_max
1020  format(I3,1x,I3,' 0    first/last line & max length for line')
      write(sss,1021) dimz, stepmx, stepmz, file_2  
1021  format(I4,1x,F5.1,1x,F5.1,' xxx  dimz, steps x/z, image factor'/
     &       A)
      write(ppp,1010) dimz, stepmx, stepmz,file_2  
1010  format(I4,1x,F5.1,1x,F5.1,' ***   dimz, step x/z, image factor'/
     &       '0     max length of line (default 0)'/
     &       A/
     &       '# Following lines:'/'# line number, machine, filename'/
     &       '# Then same number of lines with sizes for each line')  
*
*     start creation of files
*
      write(*,1050)
      write(log,1050)
1050  format(//'Characteristics of created strain files '/
     &       '----------------------------------------'/
     &     'file name',24X,'offset    dimx     dimz'//)
      do i = 1,processor
*
*     building files names: generic_name//line_number
*
          line = line_min + i -1
          do j = SIZE,1,-1             
              if(filestr(j:j).ne.' ') goto 10
          enddo
          stop 'Error generic file name to short'
10        continue
          write(unit=file_1,fmt=2001)filestr(1:j),line
2001      format(A,I3.3)
          open (unit=iin,form='unformatted', status='new',
     &          file=file_1, iostat=ir)
          if(ir.ne.0) then
              write(*,*) 'Cannot access strain value file ',file_1
              write(*,*) 'probably file already exists'
              stop 'abnormal end  open file'
          endif
*
*    compute derivatives
*
          line = line_min + i -1
          ir = strain_create(strain_val,dimz,dimx,stepz,
     &         stepx,stepy,epais,xdeb,xfin,line)
          if (ir.ne.0) then
              write(*,*) 'error creation file # ',processor
              stop 'abnormal end strain_create'
          endif
*
*    write file
*
          do iz = 1,2*dimz
              write(iin) (strain_val(ix,iz),ix=1,dimx)
          enddo
          close(unit=iin)
          offset=xmax - dimx
          write(*,2010) file_1,offset,dimx,dimz
          write(log,2010) file_1,offset,dimx,dimz
2010  format(A32,'  ',I4,'     ',I4,'     ',I4) 
          write(ppp,1011) line,file_1
1011  format(I3,' machine_name ',A32)  
*
*     Array out contains data to be printed in unit ppp later
*                   
          write(sss,1022) offset,dimx
1022      format(I3,1x,I4,'   offset, dimx')
          out(i,1) = offset
          out(i,2) = dimx
      enddo
      write(ppp,1012)
1012  format('@  end of file description. Do not remove this line')
      do i = 1,processor
         write(ppp,1013)out(i,1),out(i,2)
1013     format(I3,1x,I4)
      enddo
*
      close(unit=log)     
      close(unit=ppp)
      close(unit=sss)
      stop 'normal end'
      end
